<?php

namespace Drupal\testsuite\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Render\Markup;
use Drupal\testsuite\LoadResourceService;
use Drupal\testsuite\ShellExecService;
use Drupal\testsuite\TestMenuService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Returns responses for test reports routes.
 */
class TestSuiteController extends ControllerBase {

  /**
   * Load Resource Service.
   *
   * @var \Drupal\testsuite\LoadResourceService
   */
  protected $loadResourceService;

  /**
   * Test Menu Service.
   *
   * @var \Drupal\testsuite\TestMenuService
   */
  protected $testMenuService;

  /**
   * ShellExec Service.
   *
   * @var \Drupal\testsuite\ShellExecService
   */
  protected $shellExecService;

  /**
   * The array of loaded files.
   *
   * @var array The array of files to create Resource with
   */
  protected $loadedFiles = [];

  /**
   * Constructs a new TestSuiteController.
   *
   * @param object $loadResourceService
   *   Drupal\testsuite\LoadResourceService.
   * @param object $testMenuService
   *   Drupal\testsuite\TestMenuService.
   * @param object $shellExecService
   *   Drupal\testsuite\ShellExecService.
   */
  public function __construct(
    LoadResourceService $loadResourceService,
    TestMenuService $testMenuService,
    ShellExecService $shellExecService
  ) {
    $this->testMenuService = $testMenuService;
    $this->loadResourceService = $loadResourceService;
    $this->shellExecService = $shellExecService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('testsuite.load_resource.service'),
      $container->get('testsuite.test_menu.service'),
      $container->get('testsuite.shell_exec.service')
    );
  }

  /**
   * Builds the report for custom tests.
   *
   * @param object $request
   *   Symfony\Component\HttpFoundation\Request.
   *
   * @return array
   *   The page as a render array.
   */
  public function customReport(Request $request) {
    $output['test_menu'] = $this->testMenuService->buildTestsMenu();
    foreach ($this->loadResourceService->getResource('report', 'custom', 'testsuite', 'all', 'all') as $file) {
      $className = basename($file, '.php');
      $class = '\Drupal\custom_tests\Tests\\' . $className;
      require_once $file;
      if (class_exists($class)) {
        $obj = new $class();
        $this->loadedFiles[$className] = [
          'file' => $file,
          'moduleName' => $obj->getModuleName(),
          'name' => $obj->getName(),
          'description' => $obj->getDescription(),
          'class' => '\Drupal\custom_tests\Tests\\' . $className,
        ];
      }
    }

    $header = [
      $this->t('Name'),
      $this->t('Results'),
    ];

    $result = [];
    foreach ($this->loadedFiles as $className => $values) {
      $results = $request->getSession()->get($className);
      if ($results) {
        $result[] = [
          'name' => $values['name'],
          'results' => $results,
        ];
      }
    }

    $rows = [];

    foreach ($result as $dblog) {
      $rows[] = [
        $dblog['name'],
        $dblog['results'],
      ];
    }

    if (empty($rows)) {
      $rows[] = [[
        'Type' => $this->t('No report messages available.'),
        'colspan' => 3,
      ],
      ];
    }

    $output['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#attributes' => [
        'class' => [
          'test-suite-table',
        ],
      ],
    ];

    return $output;
  }

  /**
   * Builds the report for phpunit tests.
   *
   * @param object $request
   *   Symfony\Component\HttpFoundation\Request.
   *
   * @return array
   *   The page as a render array.
   */
  public function phpUnitReport(Request $request) {
    $output['top_menu'] = $this->testMenuService->buildTestsMenu();
    $sessionArea = $request->getSession()->get('area');
    $sessionModule = $request->getSession()->get('module');
    $sessionDirectory = $request->getSession()->get('directory');
    $this->loadedFiles = $this->loadResourceService->getResource('report', 'phpunit', $sessionArea, $sessionModule, $sessionDirectory);

    $classArray = [];
    $errorArray = [];

    foreach ($this->loadedFiles as $module => $statement) {
      $results = $this->shellExecService->getData($statement);
      if ($results == NULL) {
        $result[] = [
          'module' => 'Error',
          'name' => 'No permissions',
          'results' => Markup::create('<p>You do not have permissions to create files or you have not added all the packages you need to run PHPUnit. Run sudo chown -R www-data:www-data /your/root or follow the directions for loading the packages via composer at <a href="https://www.drupal.org/project/testsuite">Test Suite</a></p>'),
        ];
      }
      else {
        $classArray = $this->shellExecService->getReportMore($module, $results);
        if ($this->shellExecService->ifErrors($results)) {
          $errorArray = $this->shellExecService->getErrorArray($module, $results);
        }

        $result[] = [
          'module' => $module,
          'name' => $statement,
          'results' => Markup::create($this->shellExecService->getReportHeading($results)),
        ];
      }
    }

    $table = '<table class="test-suite-table responsive-enabled">';
    $table .= '<thead><tr><th>Name</th><th>Results</th><th>Actions</th></tr></thead>';

    for ($i = 0; $i < count($result); $i++) {
      $table .= '<tr class="test-suite-test-report-rows">';
      $table .= '<td>' . $result[$i]['name'] . '</td>';
      $table .= '<td width="50%">' . $result[$i]['results'] . '</td>';
      $table .= '<td><a href="#" class="button button--primary test-suite-table-row-show-button">Show Report</a></td>';
      $table .= '</tr>';
      if (isset($classArray[$result[$i]['module']])) {
        $table .= '<tr class="test-suite-test-report-table-rows">';
        $table .= '<td class="test-suite-test-report-table-columns" colspan="3">';
        $table .= '<table class="test-suite-table responsive-enabled">';
        $table .= '<thead><tr><th>Class Name</th><th>Tests Ran</th><th>Results</th></tr></thead>';
        for ($j = 0; $j < count($classArray[$result[$i]['module']]); $j++) {

          $table .= '<tr>';
          $table .= '<td>' . $classArray[$result[$i]['module']][$j]['class'] . '</td>';
          $table .= '<td>' . $classArray[$result[$i]['module']][$j]['count'] . '</td>';
          $table .= '<td>Tested.</td>';
          $table .= '</tr>';

        }
        if (isset($errorArray[$result[$i]['module']])) {
          for ($j = 0; $j < count($errorArray[$result[$i]['module']]); $j++) {

            $table .= '<tr>';
            $table .= '<td colspan="3">' . $errorArray[$result[$i]['module']][$j] . '</td>';
            $table .= '</tr>';

          }
        }
        $table .= '</table>';
        $table .= '</td></tr>';
      }
    }

    if (empty($result)) {
      $table .= '<tr><td colspan="3">There are no results yet.</td></tr>';
    }
    $table .= '</table>';

    $output['table'] = [
      '#type' => 'markup',
      '#markup' => $table,
    ];
    return $output;
  }

  /**
   * Builds the report for phpunit tests that are accessable through links.
   *
   * @param string $area
   *   The area of the module. Core, contrib or custom.
   * @param string $module
   *   The module.
   * @param string $directory
   *   The phpunit test directory. Unit, Functional, Kernel etc.
   * @param string $file
   *   The phpunit test file name.
   *
   * @return array
   *   The page as a render array.
   */
  public function phpUnitFileReport($area, $module, $directory, $file = NULL) {
    $output['top_menu'] = $this->testMenuService->buildTestsMenu();
    if ($file == NULL) {
      $statement = $this->loadResourceService->getStatement($area, $module, $directory);
    }
    else {
      $statement = $this->loadResourceService->getFileStatement($area, $module, $directory, $file);
    }

    $classArray = [];
    $errorArray = [];

    $results = $this->shellExecService->getData($statement);
    if ($results == NULL) {
      $result[] = [
        'module' => 'Error',
        'name' => 'No permissions',
        'results' => Markup::create('<p>You do not have permissions to create files or you have not added all the packages you need to run PHPUnit. Run sudo chown -R www-data:www-data /your/root or follow the directions for loading the packages via composer at <a href="https://www.drupal.org/project/testsuite">Test Suite</a></p>'),
      ];
    }
    else {
      $classArray = $this->shellExecService->getReportMore($module, $results);
      if ($this->shellExecService->ifErrors($results)) {
        $errorArray = $this->shellExecService->getErrorArray($module, $results);
      }

      $result[] = [
        'module' => $module,
        'name' => $statement,
        'results' => Markup::create($this->shellExecService->GetReportHeading($results)),
      ];
    }

    $table = '<table class="test-suite-table responsive-enabled">';
    $table .= '<thead><tr><th>Name</th><th>Results</th><th>Actions</th></tr></thead>';

    for ($i = 0; $i < count($result); $i++) {
      $table .= '<tr class="test-suite-test-report-rows">';
      $table .= '<td>' . $result[$i]['name'] . '</td>';
      $table .= '<td width="50%">' . $result[$i]['results'] . '</td>';
      $table .= '<td><a href="#" class="button button--primary test-suite-table-row-show-button">Show Report</a></td>';
      $table .= '</tr>';
      if (isset($classArray[$result[$i]['module']])) {
        $table .= '<tr class="test-suite-test-report-table-rows">';
        $table .= '<td class="test-suite-test-report-table-columns" colspan="3">';
        $table .= '<table class="test-suite-table responsive-enabled">';
        $table .= '<thead><tr><th>Class Name</th><th>Tests Ran</th><th>Results</th></tr></thead>';
        for ($j = 0; $j < count($classArray[$result[$i]['module']]); $j++) {

          $table .= '<tr>';
          $table .= '<td>' . $classArray[$result[$i]['module']][$j]['class'] . '</td>';
          $table .= '<td>' . $classArray[$result[$i]['module']][$j]['count'] . '</td>';
          $table .= '<td>Tested.</td>';
          $table .= '</tr>';

        }
        if (isset($errorArray[$result[$i]['module']])) {
          for ($j = 0; $j < count($errorArray[$result[$i]['module']]); $j++) {

            $table .= '<tr>';
            $table .= '<td colspan="3">' . $errorArray[$result[$i]['module']][$j] . '</td>';
            $table .= '</tr>';

          }
        }
        $table .= '</table>';
        $table .= '</td></tr>';
      }
    }

    if (empty($result)) {
      $table .= '<tr><td colspan="3">There are no results yet.</td></tr>';
    }
    $table .= '</table>';

    $output['table'] = [
      '#type' => 'markup',
      '#markup' => $table,
    ];
    return $output;
  }

}
