<?php

namespace Drupal\testsuite\Form;

use Drupal\Core\Database\Connection;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\testsuite\InitializerService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Defines a form that configures forms module settings.
 */
class ConfigurationForm extends ConfigFormBase {

  /**
   * Initializer Service.
   *
   * @var \Drupal\testsuite\InitializerService
   */
  protected $initializerService;

  /**
   * Drupal\Core\Messenger\Messenger.
   *
   * @var object
   */
  protected $messenger;

  /**
   * Request object.
   *
   * @var object
   */
  protected $request;

  /**
   * Database object.
   *
   * @var object
   */
  protected $connection;

  /**
   * Absolute path to browser output directory.
   *
   * @var string
   */
  protected $browserOutputDirectory;

  /**
   * ConfigurationForm constructor.
   *
   * @param \Drupal\testsuite\InitializerService $initializerService
   *   The initializer object.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   The messenger object.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack.
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection.
   */
  public function __construct(
    InitializerService $initializerService,
    Messenger $messenger,
    RequestStack $requestStack,
    Connection $connection
  ) {
    $this->initializerService = $initializerService;
    $this->messenger = $messenger;
    $this->request = $requestStack->getCurrentRequest();
    $this->connection = $connection;
    if ($this->initializerService->isWindows()) {
      $path = dirname(dirname(dirname(dirname(dirname(__DIR__)))));
      $this->browserOutputDirectory = $path . '\sites\simpletest\browser_output';
    }
    else {
      $path = dirname(dirname(dirname(dirname(dirname(__DIR__)))));
      $this->browserOutputDirectory = $path . '/sites/simpletest/browser_output';
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('testsuite.initializer.service'),
      $container->get('messenger'),
      $container->get('request_stack'),
      $container->get('database')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'testsuite_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'testsuite.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('testsuite.settings');

    $form['testsuite_mysql_storage_type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Host and Db connection variables'),
      '#options' => [
        $this->t('Test Suite Settings'),
        $this->t('Drupal settings.php'),
        $this->t('.env file'),
      ],
      '#default_value' => $config->get('testsuite_mysql_storage_type') ? $config->get('testsuite_mysql_storage_type') : 0,
      '#attributes' => [
        'id' => 'test-suite-mysql-storage-type',
      ],
    ];

    $form['database'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Host and Database Credentials'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      '#states' => [
        'visible' => [
          [
            [':input[id="test-suite-mysql-storage-type"]' => ['value' => 0]],
            'or',
            [':input[id="test-suite-mysql-storage-type"]' => ['value' => 2]],
          ],
        ],
      ],
    ];

    $form['database']['testsuite_simple_base_url'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('SIMPLETEST_BASE_URL'),
      '#description' => $this->t('Example SIMPLETEST_BASE_URL<br />
                value: http://localhost'),
      '#default_value' => $config->get('testsuite_simple_base_url') ? $config->get('testsuite_simple_base_url') : 'http://local.site',
    ];
    $form['database']['testsuite_mysql_host'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('Mysql Host'),
      '#default_value' => $config->get('testsuite_mysql_host') ? $config->get('testsuite_mysql_host') : 'localhost',
    ];
    $form['database']['testsuite_mysql_username'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('Mysql Username'),
      '#default_value' => $config->get('testsuite_mysql_username') ? $config->get('testsuite_mysql_username') : 'root',
    ];
    $form['database']['testsuite_mysql_password'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Mysql Password'),
      '#default_value' => $config->get('testsuite_mysql_password') ? $config->get('testsuite_mysql_password') : '',
    ];
    $form['database']['testsuite_mysql_port'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('Mysql Port'),
      '#default_value' => $config->get('testsuite_mysql_port') ? $config->get('testsuite_mysql_port') : '3306',
    ];
    $form['database']['testsuite_mysql_database_name'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('Mysql Database Name'),
      '#default_value' => $config->get('testsuite_mysql_database_name') ? $config->get('testsuite_mysql_database_name') : 'localdb',
    ];

    $form['testsuite_browser_output_directory'] = [
      '#type' => 'textfield',
      '#title' => $this->t('BROWSERTEST_OUTPUT_DIRECTORY'),
      '#description' => $this->t('Example BROWSERTEST_OUTPUT_DIRECTORY<br />
                value: /path/to/webroot/sites/simpletest/browser_output.<br />
                BROWSERTEST_OUTPUT_DIRECTORY is used as the directory where<br />
                the output data will be saved by PHPUnit and needs to be an<br />
                absolute local path.'),
      '#default_value' => $config->get('testsuite_browser_output_directory') ? $config->get('testsuite_browser_output_directory') : $this->browserOutputDirectory,
    ];

    $form['testsuite_browser_output_base_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('BROWSERTEST_OUTPUT_BASE_URL'),
      '#description' => $this->t('By default, browser tests will output links that use the base URL set<br />
                in SIMPLETEST_BASE_URL. However, if your SIMPLETEST_BASE_URL is an internal<br />
                path (such as may be the case in a virtual or Docker-based environment),<br />
                you can set the base URL used in the browser test output links to something<br />
                reachable from your host machine here. This will allow you to follow them<br />
                directly and view the output.'),
      '#default_value' => $config->get('testsuite_browser_output_base_url'),
    ];

    $form['testsuite_disable_deprecation_testing'] = [
      '#type' => 'radios',
      '#title' => $this->t('Disable deprecation testing.'),
      '#description' => $this->t("Deprecation testing is managed through Symfony's PHPUnit Bridge.<br />
                The environment variable SYMFONY_DEPRECATIONS_HELPER is used to<br />
                configure the behaviour of the deprecation tests. See <a href=\"https://symfony.com/doc/current/components/phpunit_bridge.html#configuration\" target=\"_blank\">configuration</a><br />
                Drupal core's testing framework is setting this variable to its defaults.<br />
                Projects with their own requirements need to manage this variable<br />
                explicitly. To disable deprecation testing completely select Disable.<br />
                To set a value for deprecation testing select Configure."),
      '#options' => [
        0 => $this->t('No change'),
        1 => $this->t('Disable'),
        2 => $this->t('Configure'),
      ],
      '#default_value' => $config->get('testsuite_disable_deprecation_testing') ? $config->get('testsuite_disable_deprecation_testing') : 0,
      '#attributes' => [
        'id' => 'test-suite-disable-deprecation',
      ],
    ];

    $form['testsuite_configure_deprecation_helper'] = [
      '#type' => 'textfield',
      '#title' => $this->t('SYMFONY_DEPRECATIONS_HELPER'),
      '#default_value' => $config->get('testsuite_configure_deprecation_helper'),
      '#states' => [
        'visible' => [
          ':input[id="test-suite-disable-deprecation"]' => ['value' => 2],
        ],
      ],
    ];

    $form['testsuite_mink_driver_class'] = [
      '#type' => 'textfield',
      '#title' => $this->t('MINK_DRIVER_CLASS'),
      '#description' => $this->t("Example for changing the driver class for mink tests MINK_DRIVER_CLASS<br />
                value: 'Drupal\FunctionalJavascriptTests\DrupalSelenium2Driver'"),
      '#default_value' => $config->get('testsuite_mink_driver_class'),
    ];

    $form['testsuite_mink_driver_args'] = [
      '#type' => 'textfield',
      '#title' => $this->t('MINK_DRIVER_ARGS'),
      '#description' => $this->t("Example for changing the driver args to mink tests<br />
                MINK_DRIVER_ARGS value: '[\"http://127.0.0.1:8510\"]'"),
      '#default_value' => $config->get('testsuite_mink_driver_args'),
    ];

    $form['testsuite_mink_driver_args_phantomjs'] = [
      '#type' => 'textfield',
      '#title' => $this->t('MINK_DRIVER_ARGS_PHANTOMJS'),
      '#description' => $this->t("Example for changing the driver args to phantomjs tests<br />
                 MINK_DRIVER_ARGS_PHANTOMJS value: '[\"http://127.0.0.1:8510\"]'"),
      '#default_value' => $config->get('testsuite_mink_driver_args_phantomjs'),
    ];

    $form['testsuite_mink_driver_args_webdriver'] = [
      '#type' => 'textfield',
      '#title' => $this->t('MINK_DRIVER_ARGS_WEBDRIVER'),
      '#description' => $this->t("Example for changing the driver args to webdriver tests<br />
                MINK_DRIVER_ARGS_WEBDRIVER value: <br />
                '[\"chrome\", { \"chromeOptions\": { \"w3c\": false } }, \"http://localhost:4444/wd/hub\"]'<br />
                For using the Firefox browser, replace \"chrome\" with \"firefox\""),
      '#default_value' => $config->get('testsuite_mink_driver_args_webdriver'),
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $data = [];
    $settings = $this->config('testsuite.settings');
    $settings->set('testsuite_mysql_storage_type', $form_state->getValue('testsuite_mysql_storage_type'))
      ->set('testsuite_simple_base_url', $form_state->getValue('testsuite_simple_base_url'))
      ->set('testsuite_mysql_host', $form_state->getValue('testsuite_mysql_host'))
      ->set('testsuite_mysql_username', $form_state->getValue('testsuite_mysql_username'))
      ->set('testsuite_mysql_port', $form_state->getValue('testsuite_mysql_port'))
      ->set('testsuite_mysql_database_name', $form_state->getValue('testsuite_mysql_database_name'))
      ->set('testsuite_mysql_password', $form_state->getValue('testsuite_mysql_password'))
      ->set('testsuite_browser_output_directory', $form_state->getValue('testsuite_browser_output_directory'))
      ->set('testsuite_browser_output_base_url', $form_state->getValue('testsuite_browser_output_base_url'))
      ->set('testsuite_disable_deprecation_testing', $form_state->getValue('testsuite_disable_deprecation_testing'))
      ->save();
    if ($form_state->getValue('testsuite_mysql_storage_type') == 1) {
      $connection_info = $this->connection->getConnectionOptions();
      $data['mysql_host'] = $connection_info['host'];
      $data['mysql_username'] = $connection_info['username'];
      $data['mysql_password'] = $connection_info['password'];
      $data['mysql_port'] = $connection_info['port'];
      $data['mysql_database_name'] = $connection_info['database'];
      $data['simple_base_url'] = $this->request->getSchemeAndHttpHost();
    }
    else {
      $data['mysql_host'] = $form_state->getValue('testsuite_mysql_host');
      $data['mysql_username'] = $form_state->getValue('testsuite_mysql_username');
      $data['mysql_password'] = $form_state->getValue('testsuite_mysql_password');
      $data['mysql_port'] = $form_state->getValue('testsuite_mysql_port');
      $data['mysql_database_name'] = $form_state->getValue('testsuite_mysql_database_name');
      $data['simple_base_url'] = $form_state->getValue('testsuite_simple_base_url');
    }
    $data['browser_output_directory'] = $form_state->getValue('testsuite_browser_output_directory');
    $data['browser_output_base_url'] = $form_state->getValue('testsuite_browser_output_base_url');
    $data['disable_deprecation_testing'] = $form_state->getValue('testsuite_disable_deprecation_testing');
    $data['disable_deprecation_testing'] = $form_state->getValue('testsuite_disable_deprecation_testing');
    if ($form_state->getValue('testsuite_disable_deprecation_testing') == 2) {
      $settings->set('testsuite_configure_deprecation_helper', $form_state->getValue('testsuite_configure_deprecation_helper'))->save();
      $data['configure_deprecation_helper'] = $form_state->getValue('testsuite_configure_deprecation_helper');
    }
    $settings->set('testsuite_mink_driver_class', $form_state->getValue('testsuite_mink_driver_class'))
      ->set('testsuite_mink_driver_args', $form_state->getValue('testsuite_mink_driver_args'))
      ->set('testsuite_mink_driver_args_phantomjs', $form_state->getValue('testsuite_mink_driver_args_phantomjs'))
      ->set('testsuite_mink_driver_args_webdriver', $form_state->getValue('testsuite_mink_driver_args_webdriver'))
      ->save();
    $data['mink_driver_class'] = $form_state->getValue('testsuite_mink_driver_class');
    $data['mink_driver_args'] = $form_state->getValue('testsuite_mink_driver_args');
    $data['mink_driver_args_phantomjs'] = $form_state->getValue('testsuite_mink_driver_args_phantomjs');
    $data['mink_driver_args_webdriver'] = $form_state->getValue('testsuite_mink_driver_args_webdriver');

    if ($this->initializerService->ifCoreIsWritable()) {
      if (!$this->initializerService->ifPhpUnitXmlExists()) {
        $this->initializerService->copyPhpunitFile();
      }
      $this->initializerService->rebuildPhpUnitXml($data);
    }
    else {
      $this->messenger->addStatus('The phpunit.xml file was unable to be created. This is most likely due to permissions on the core folder.');
    }
    if ($form_state->getValue('testsuite_browser_output_directory') != "") {
      if ($this->initializerService->ifSitesSimpletestBrowserOutputPath($form_state->getValue('testsuite_browser_output_directory'))) {
        $this->initializerService->createBrowserOutputFolderIfNotExists($form_state->getValue('testsuite_browser_output_directory'));
      }
    }

    parent::submitForm($form, $form_state);
  }

}
