<?php

namespace Drupal\testsuite\Form;

use Drupal\Core\Extension\ModuleHandler;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\testsuite\LoadResourceService;
use Drupal\testsuite\TestMenuService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure example settings for this site.
 */
class CustomTestsForm extends ConfigFormBase {

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'testsuite.custom';

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandler
   */
  protected $moduleHandler;

  /**
   * Builds the menu.
   *
   * @var \Drupal\testsuite\TestMenuService
   */
  protected $testMenuService;

  /**
   * The array of files to create Resource with.
   *
   * @var array
   */
  protected $loadedFiles = [];

  /**
   * If there are test.
   *
   * @var bool
   */
  protected $testsFound = TRUE;

  /**
   * CustomTestsForm constructor.
   *
   * @param \Drupal\testsuite\LoadResourceService $loadResourceService
   *   Creates the phpunit statements.
   * @param \Drupal\testsuite\TestMenuService $testMenuService
   *   Creates the phpunit menu.
   * @param \Drupal\Core\Extension\ModuleHandler $moduleHandler
   *   The module handler.
   */
  public function __construct(
    LoadResourceService $loadResourceService,
    TestMenuService $testMenuService,
    ModuleHandler $moduleHandler
  ) {
    $this->testMenuService = $testMenuService;
    $files = $loadResourceService->getResource('form', 'custom', 'all', 'all', 'all');
    if ($moduleHandler->moduleExists('custom_tests')) {
      if (count($files) > 0) {
        foreach ($files as $file) {
          $className = basename($file, '.php');
          $class = '\Drupal\custom_tests\Tests\\' . $className;
          require_once $file;
          if (class_exists($class)) {
            $obj = new $class();
            if ($moduleHandler->moduleExists($obj->getModuleName()) || $obj->getModuleName() == '') {
              $this->loadedFiles[$className] = [
                'file' => $file,
                'moduleName' => $obj->getModuleName(),
                'name' => $obj->getName(),
                'description' => $obj->getDescription(),
                'class' => '\Drupal\custom_tests\Tests\\' . $className,
              ];
            }
          }
        }
      }
      elseif ($moduleHandler->moduleExists('custom_tests')) {
        $this->testsFound = FALSE;
        $this->loadedFiles['tests_not_found'] = [
          'name' => 'No custom tests found.',
        ];
      }
      else {
        $this->testsFound = FALSE;
        $this->loadedFiles['tests_not_found'] = [
          'name' => 'Please enable custom_tests module to create custom tests.',
        ];
      }
    }
    else {
      $this->testsFound = FALSE;
      $this->loadedFiles['custom_tests_not_found'] = [
        'name' => 'Please copy the custom_tests module into the modules/custom folder, rename custom_tests.info.yml.txt to custom_tests.info.yml and enable the module to create and use custom tests.',
      ];
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('testsuite.load_resource.service'),
      $container->get('testsuite.test_menu.service'),
      $container->get('module_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'testsuite_custom_tests_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $top_menu = $this->testMenuService->buildTestsMenu();

    if ($this->testsFound) {
      $container['submit'] = [
        '#type' => 'submit',
        '#attributes' => [
          'class' => [
            'button', 'button--primary',
            'test-suite-top-build-button',
          ],
        ],
        '#value' => $this->t('Build Report'),
      ];
    }

    foreach ($this->loadedFiles as $className => $values) {
      if ($className == 'custom_tests_not_found') {
        $container[$className] = [
          '#type' => 'markup',
          '#markup' => '<p>' . $values['name'] . '</p>',
        ];
      }
      elseif ($className == 'tests_not_found') {
        $container[$className] = [
          '#type' => 'markup',
          '#markup' => '<p>' . $values['name'] . '</p>',
        ];
      }
      else {
        $container[$className] = [
          '#type' => 'checkbox',
          '#title' => $values['name'],
          '#attributes' => [
            'id' => $className,
            'name' => $className,
          ],
          '#default_value' => TRUE,
        ];
      }
    }

    if ($this->testsFound) {
      $container['submit_bottom'] = [
        '#type' => 'submit',
        '#attributes' => [
          'class' => ['button', 'button--primary'],
        ],
        '#value' => $this->t('Build Report'),
      ];
    }

    $form['container'] = [
      '#type' => 'html_tag',
      '#tag' => 'div',
      '#attributes' => [
        'class' => ['test-suite-container'],
      ],
      'children' => [
        $top_menu,
        [
          '#type' => 'html_tag',
          '#tag' => 'div',
          '#attributes' => ['class' => ['test-suite-custom-row']],
          'children' => [
            [
              '#type' => 'html_tag',
              '#tag' => 'div',
              '#attributes' => ['class' => ['test-suite-content']],
              'child' => $container,
            ],
          ],
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    foreach ($this->loadedFiles as $className => $values) {
      if ($form_state->hasValue($className) && $form_state->getValue($className) == TRUE) {
        require_once $values['file'];
        if (class_exists($values['class'])) {
          $obj = new $values['class']();
          $request = $this->getRequest();
          $results = $obj->runTest($request);
          $request->getSession()->set($className, $results);
          $form_state->setRedirect('testsuite.custom_report');
        }
      }
    }
  }

}
