<?php

namespace Drupal\testsuite\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\testsuite\LoadResourceService;
use Drupal\testsuite\TestMenuService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure example settings for this site.
 */
class PhpUnitTestsForm extends ConfigFormBase {

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'testsuite.phpunit';

  /**
   * Load Resource Service.
   *
   * @var \Drupal\testsuite\LoadResourceService
   */
  protected $loadResourceService;

  /**
   * Test Menu Service.
   *
   * @var \Drupal\testsuite\TestMenuService
   */
  protected $testMenuService;

  /**
   * The array of files to create Resource with.
   *
   * @var array
   */
  protected $loadedFiles = [];

  /**
   * PhpUnitTestsForm constructor.
   *
   * @param \Drupal\testsuite\LoadResourceService $loadResourceService
   *   Load resource Service.
   * @param \Drupal\testsuite\TestMenuService $testMenuService
   *   Test Menu Service.
   */
  public function __construct(
    LoadResourceService $loadResourceService,
    TestMenuService $testMenuService
  ) {
    $this->testMenuService = $testMenuService;
    $this->loadResourceService = $loadResourceService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('testsuite.load_resource.service'),
      $container->get('testsuite.test_menu.service'),
      $container->get('testsuite.initializer.service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'testsuite_phpunit_tests_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $area = 'contrib', $module = 'testsuite', $directory = 'Unit') {
    $top_menu = $this->testMenuService->buildTestsMenu();
    $module_menu = $this->testMenuService->buildModuleMenu();
    $this->loadedFiles = $this->loadResourceService->getResource('form', 'phpunit', $area, $module, $directory);

    $container['submit'] = [
      '#type' => 'submit',
      '#attributes' => [
        'class' => ['button', 'button--primary', 'test-suite-top-build-button'],
      ],
      '#value' => $this->t('Build Report'),
    ];

    foreach ($this->loadedFiles as $loadedModule => $statement) {
      $files = $this->loadResourceService->getFiles($area, $module, $directory);
      // dd($files);
      $statementString = $statement;
      foreach ($files as $file) {
        $statementString .= '<br /><a href="' . $file . '" target="_blank">' . $file . '</a>';
      }
      $container[$loadedModule] = [
        '#type' => 'checkbox',
        '#title' => $statementString,
        '#attributes' => [
          'id' => $loadedModule,
          'name' => $loadedModule,
        ],
        '#default_value' => TRUE,
      ];
    }

    $container['area'] = [
      '#type' => 'hidden',
      '#value' => $area,
    ];

    $container['module'] = [
      '#type' => 'hidden',
      '#value' => $module,
    ];

    $container['directory'] = [
      '#type' => 'hidden',
      '#value' => $directory,
    ];

    $container['submit_bottom'] = [
      '#type' => 'submit',
      '#attributes' => [
        'class' => ['button', 'button--primary'],
      ],
      '#value' => $this->t('Build Report'),
    ];

    $form['container'] = [
      '#type' => 'html_tag',
      '#tag' => 'div',
      '#attributes' => [
        'id' => 'test-suite-ajax-response-container',
        'class' => ['test-suite-container'],
      ],
      'children' => [
        $top_menu,
        [
          '#type' => 'html_tag',
          '#tag' => 'div',
          '#attributes' => ['class' => ['test-suite-row']],
          'children' => [
            [
              '#type' => 'html_tag',
              '#tag' => 'div',
              '#attributes' => ['class' => ['test-suite-menu']],
              'child' => $module_menu,
            ],
            [
              '#type' => 'html_tag',
              '#tag' => 'div',
              '#attributes' => ['class' => ['test-suite-content']],
              'child' => $container,
            ],
          ],
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    foreach ($this->loadedFiles as $module => $statement) {
      if ($form_state->hasValue($module) && $form_state->getValue($module) == TRUE) {
        $request = $this->getRequest();
        if ($form_state->hasValue('directory')) {
          $request->getSession()->set('directory', $form_state->getValue('directory'));
        }
        if ($form_state->hasValue('module')) {
          $request->getSession()->set('module', $form_state->getValue('module'));
        }
        if ($form_state->hasValue('area')) {
          $request->getSession()->set('area', $form_state->getValue('area'));
        }
        $form_state->setRedirect('testsuite.phpunit_report');
      }
    }
  }

}
