<?php

namespace Drupal\testsuite;

use Drupal\Core\Messenger\Messenger;

/**
 * Test Suite Services.
 */
class InitializerService extends BaseService {

  /**
   * Drupal\Core\Messenger\Messenger.
   *
   * @var object
   */
  protected $messenger;

  /**
   * The path to the phpunit.xml.dist file.
   *
   * @var string
   */
  protected $phpunitXmlDistPath;

  /**
   * The path to the phpunit.xml file.
   *
   * @var string
   */
  protected $phpunitXmlPath;

  /**
   * The path to the core folder.
   *
   * @var string
   */
  protected $phpunitCore;

  /**
   * The base path.
   *
   * @var string
   */
  protected $simpleTestPath;

  /**
   * Initializer Service constructor.
   */
  public function __construct(Messenger $messenger) {
    $this->messenger = $messenger;
    if ($this->isWindows()) {
      $this->phpunitCore = dirname(dirname(dirname(dirname(__DIR__)))) . '\core';
      $this->simpleTestPath = dirname(dirname(dirname(dirname(__DIR__)))) . '\sites\simpletest';
      $this->phpunitXmlDistPath = dirname(dirname(dirname(dirname(__DIR__)))) . '\core\phpunit.xml.dist';
      $this->phpunitXmlPath = dirname(dirname(dirname(dirname(__DIR__)))) . '\core\phpunit.xml';
    }
    else {
      $this->phpunitCore = dirname(dirname(dirname(dirname(__DIR__)))) . '/core';
      $this->simpleTestPath = dirname(dirname(dirname(dirname(__DIR__)))) . '/sites/simpletest';
      $this->phpunitXmlDistPath = dirname(dirname(dirname(dirname(__DIR__)))) . '/core/phpunit.xml.dist';
      $this->phpunitXmlPath = dirname(dirname(dirname(dirname(__DIR__)))) . '/core/phpunit.xml';
    }
  }

  /**
   * Checks if phpunit.xml exists in core folder.
   *
   * @return bool
   *   True or false.
   */
  public function ifPhpUnitXmlExists() {
    if (file_exists($this->phpunitXmlPath)) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Checks to see if the core folder is writable.
   */
  public function ifCoreIsWritable() {
    if (is_writable($this->phpunitCore)) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Creates the phpunit.xml if it does not exist.
   */
  public function copyPhpunitFile() {
    try {
      copy($this->phpunitXmlDistPath, $this->phpunitXmlPath);
      $this->messenger->addStatus('A new phpunit.xml file was created.');
    }
    catch (\Exception $e) {
      $this->messenger->addStatus('An error has occured creating phpunit.xml file. Please check the logs for more information.');
    }
  }

  /**
   * Checks to see if path equals sites/simpletest/browser_output.
   */
  public function ifSitesSimpletestBrowserOutputPath($outputDir) {
    if ($this->isWindows()) {
      $parts = explode('\\', $outputDir);
      $parts = array_reverse($parts);
      if ($parts[0] == "browser_output" && $parts[1] == "simpletest" && $parts[2] == "sites") {
        return TRUE;
      }
    }
    else {
      $parts = explode('/', $outputDir);
      $parts = array_reverse($parts);
      if ($parts[0] == "browser_output" && $parts[1] == "simpletest" && $parts[2] == "sites") {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Checks to see if the browser_output folder is writable.
   */
  private function ifBrowserOutputIsWritable($outputDir) {
    if (is_writable($outputDir)) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Creates the browser_output file if it does not exist.
   */
  public function createBrowserOutputFolderIfNotExists($outputDir) {
    if (!is_dir($this->simpleTestPath)) {
      mkdir($this->simpleTestPath, 0777);
      $this->messenger->addStatus('The simpletest directory was created in sites\ directory.');
    }
    if (!is_dir($outputDir)) {
      mkdir($outputDir, 0777);
      $this->messenger->addStatus('The browser_output directory was created in sites\simpletest directory.');
    }
    if (!is_dir($outputDir) && !$this->ifBrowserOutputIsWritable($outputDir)) {
      $this->messenger->addStatus('The browser_output folder was unable to be created.');
    }
  }

  /**
   * Rebuilds the phpunit.xml with config variables.
   */
  public function rebuildPhpUnitXml($data) {
    $content = '<?xml version="1.0" encoding="UTF-8"?>' . "\n";
    $content .= "\n";
    $content .= '<!-- TODO set checkForUnintentionallyCoveredCode="true" once https://www.drupal.org/node/2626832 is resolved. -->' . "\n";
    $content .= '<!-- PHPUnit expects functional tests to be run with either a privileged user' . "\n";
    $content .= 'or your current system user. See core/tests/README.md and' . "\n";
    $content .= 'https://www.drupal.org/node/2116263 for details.' . "\n";
    $content .= '-->' . "\n";
    $content .= '<phpunit bootstrap="tests/bootstrap.php" colors="true"' . "\n";
    $content .= '     beStrictAboutTestsThatDoNotTestAnything="true"' . "\n";
    $content .= '     beStrictAboutOutputDuringTests="true"' . "\n";
    $content .= '     beStrictAboutChangesToGlobalState="true"' . "\n";
    $content .= '     printerClass="\Drupal\Tests\Listeners\HtmlOutputPrinter">' . "\n";
    $content .= '<php>' . "\n";
    $content .= '    <!-- Set error reporting to E_ALL. -->' . "\n";
    $content .= '    <ini name="error_reporting" value="32767"/>' . "\n";
    $content .= '    <!-- Do not limit the amount of memory tests take to run. -->' . "\n";
    $content .= '    <ini name="memory_limit" value="-1"/>' . "\n";
    $content .= '    <!-- Example SIMPLETEST_BASE_URL value: http://localhost -->' . "\n";
    $content .= '    <env name="SIMPLETEST_BASE_URL" value="' . $data['simple_base_url'] . '/"/>' . "\n";
    $content .= '    <!-- Example SIMPLETEST_DB value: mysql://username:password@localhost/databasename#table_prefix -->' . "\n";
    $content .= '    <env name="SIMPLETEST_DB" value="mysql://' . $data['mysql_username'] . ':' . $data['mysql_password'] . '@' . $data['mysql_host'] . ':' . $data['mysql_port'] . '/' . $data['mysql_database_name'] . '"/>' . "\n";
    $content .= '    <!-- Example BROWSERTEST_OUTPUT_DIRECTORY value: /path/to/webroot/sites/simpletest/browser_output -->' . "\n";
    $content .= '    <env name="BROWSERTEST_OUTPUT_DIRECTORY" value="' . $data['browser_output_directory'] . '"/>' . "\n";
    $content .= '    <!-- To have browsertest output use an alternative base URL. For example if' . "\n";
    $content .= '    SIMPLETEST_BASE_URL is an internal DDEV URL, you can set this to the' . "\n";
    $content .= '    external DDev URL so you can follow the links directly.' . "\n";
    $content .= '    -->' . "\n";
    $content .= '    <env name="BROWSERTEST_OUTPUT_BASE_URL" value="' . $data['browser_output_base_url'] . '"/>' . "\n";
    $content .= '    <!-- To disable deprecation testing completely uncomment the next line. -->' . "\n";
    if ($data['disable_deprecation_testing'] == 1) {
      $content .= '    <env name="SYMFONY_DEPRECATIONS_HELPER" value="disabled"/>' . "\n";
    }
    elseif ($data['disable_deprecation_testing'] == 2) {
      $content .= '    <env name="SYMFONY_DEPRECATIONS_HELPER" value="' . $data['configure_deprecation_helper'] . '"/>' . "\n";
    }
    else {
      $content .= '    <!-- <env name="SYMFONY_DEPRECATIONS_HELPER" value="disabled"/> -->' . "\n";
    }
    $content .= '    <!-- Example for changing the driver class for mink tests MINK_DRIVER_CLASS value: \'Drupal\FunctionalJavascriptTests\DrupalSelenium2Driver\' -->' . "\n";
    $content .= '    <env name="MINK_DRIVER_CLASS" value=\'' . $data['mink_driver_class'] . '\'/>' . "\n";
    $content .= '    <!-- Example for changing the driver args to mink tests MINK_DRIVER_ARGS value: \'["http://127.0.0.1:8510"]\' -->' . "\n";
    $content .= '    <env name="MINK_DRIVER_ARGS" value=\'' . $data['mink_driver_args'] . '\'/>' . "\n";
    $content .= '    <!-- Example for changing the driver args to phantomjs tests MINK_DRIVER_ARGS_PHANTOMJS value: \'["http://127.0.0.1:8510"]\' -->' . "\n";
    $content .= '    <env name="MINK_DRIVER_ARGS_PHANTOMJS" value=\'' . $data['mink_driver_args_phantomjs'] . '\'/>' . "\n";
    $content .= '    <!-- Example for changing the driver args to webdriver tests MINK_DRIVER_ARGS_WEBDRIVER value: \'["chrome", { "chromeOptions": { "w3c": false } }, "http://localhost:4444/wd/hub"]\' For using the Firefox browser, replace "chrome" with "firefox" -->' . "\n";
    $content .= '    <env name="MINK_DRIVER_ARGS_WEBDRIVER" value=\'' . $data['mink_driver_args_webdriver'] . '\'/>' . "\n";
    $content .= '</php>' . "\n";
    $content .= '<testsuites>' . "\n";
    $content .= '  <testsuite name="unit">' . "\n";
    $content .= '    <file>./tests/TestSuites/UnitTestSuite.php</file>' . "\n";
    $content .= '  </testsuite>' . "\n";
    $content .= '  <testsuite name="kernel">' . "\n";
    $content .= '    <file>./tests/TestSuites/KernelTestSuite.php</file>' . "\n";
    $content .= '  </testsuite>' . "\n";
    $content .= '  <testsuite name="functional">' . "\n";
    $content .= '    <file>./tests/TestSuites/FunctionalTestSuite.php</file>' . "\n";
    $content .= '  </testsuite>' . "\n";
    $content .= '  <testsuite name="functional-javascript">' . "\n";
    $content .= '    <file>./tests/TestSuites/FunctionalJavascriptTestSuite.php</file>' . "\n";
    $content .= '  </testsuite>' . "\n";
    $content .= '  <testsuite name="build">' . "\n";
    $content .= '    <file>./tests/TestSuites/BuildTestSuite.php</file>' . "\n";
    $content .= '  </testsuite>' . "\n";
    $content .= '</testsuites>' . "\n";
    $content .= '<listeners>' . "\n";
    $content .= '  <listener class="\Drupal\Tests\Listeners\DrupalListener">' . "\n";
    $content .= '  </listener>' . "\n";
    $content .= '  <!-- The Symfony deprecation listener has to come after the Drupal listener -->' . "\n";
    $content .= '  <listener class="Symfony\Bridge\PhpUnit\SymfonyTestsListener">' . "\n";
    $content .= '  </listener>' . "\n";
    $content .= '</listeners>' . "\n";
    $content .= '<!-- Filter for coverage reports. -->' . "\n";
    $content .= '<filter>' . "\n";
    $content .= '  <whitelist>' . "\n";
    $content .= '    <directory>./includes</directory>' . "\n";
    $content .= '    <directory>./lib</directory>' . "\n";
    $content .= '    <!-- Extensions can have their own test directories, so exclude those. -->' . "\n";
    $content .= '    <directory>./modules</directory>' . "\n";
    $content .= '    <exclude>' . "\n";
    $content .= '      <directory>./modules/*/src/Tests</directory>' . "\n";
    $content .= '      <directory>./modules/*/tests</directory>' . "\n";
    $content .= '    </exclude>' . "\n";
    $content .= '    <directory>../modules</directory>' . "\n";
    $content .= '    <exclude>' . "\n";
    $content .= '      <directory>../modules/*/src/Tests</directory>' . "\n";
    $content .= '      <directory>../modules/*/tests</directory>' . "\n";
    $content .= '      <directory>../modules/*/*/src/Tests</directory>' . "\n";
    $content .= '      <directory>../modules/*/*/tests</directory>' . "\n";
    $content .= '    </exclude>' . "\n";
    $content .= '    <directory>../sites</directory>' . "\n";
    $content .= '  </whitelist>' . "\n";
    $content .= '</filter>' . "\n";
    $content .= '</phpunit>' . "\n";

    try {
      $fp = fopen($this->phpunitXmlPath, "wb");
      fwrite($fp, $content);
      fclose($fp);
      $this->messenger->addStatus('The phpunit.xml was updated.');
    }
    catch (\Exception $e) {
      $this->messenger->addStatus('An error has occured updating phpunit.xml file. Please check the logs for more information.');
    }
  }

}
